from inewave.nwlistop.gttot00 import Gttot00

from tests.mocks.mock_open import mock_open
from unittest.mock import MagicMock, patch

from tests.mocks.arquivos.gttot00 import MockGttot00


def test_atributos_encontrados_gttot00():
    m: MagicMock = mock_open(read_data="".join(MockGttot00))
    with patch("builtins.open", m):
        n = Gttot00.le_arquivo("")
        assert n.submercado is not None
        assert n.submercado == "SUDESTE"
        assert n.geracao is not None
        assert n.geracao.iloc[0, 0] == 2022
        assert n.geracao.iloc[-1, -1] == 2710.5


def test_atributos_nao_encontrados_gttot00():
    m: MagicMock = mock_open(read_data="")
    with patch("builtins.open", m):
        n = Gttot00.le_arquivo("")
        assert n.submercado is None
        assert n.geracao is None


def test_eq_gttot00():
    m: MagicMock = mock_open(read_data="".join(MockGttot00))
    with patch("builtins.open", m):
        n1 = Gttot00.le_arquivo("")
        n2 = Gttot00.le_arquivo("")
        assert n1 == n2


# Não deve ter teste de diferença, visto que o atributo é
# implementado como Lazy Property.



from inewave.nwlistop.geol import Geol

from tests.mocks.mock_open import mock_open
from unittest.mock import MagicMock, patch

from tests.mocks.arquivos.geol import MockGeol


def test_atributos_encontrados_geol():
    m: MagicMock = mock_open(read_data="".join(MockGeol))
    with patch("builtins.open", m):
        n = Geol.le_arquivo("")
        assert n.pee is not None
        # assert n.pee == "cluster_NE_1"
        assert n.geracao is not None
        assert n.geracao.iloc[0, 0] == 2021
        assert n.geracao.iloc[-1, -1] == 2652.7


def test_atributos_nao_encontrados_geol():
    m: MagicMock = mock_open(read_data="")
    with patch("builtins.open", m):
        n = Geol.le_arquivo("")
        assert n.pee is None
        assert n.geracao is None


def test_eq_geol():
    m: MagicMock = mock_open(read_data="".join(MockGeol))
    with patch("builtins.open", m):
        n1 = Geol.le_arquivo("")
        n2 = Geol.le_arquivo("")
        assert n1 == n2


# Não deve ter teste de diferença, visto que o atributo é
# implementado como Lazy Property.



from inewave.nwlistop.modelos.coper import CoperAnos

from cfinterface.files.blockfile import BlockFile
import pandas as pd  # type: ignore
from typing import TypeVar, Optional


class Coper(BlockFile):
    """
    Armazena os dados das saídas referentes ao custo total de operação
    de cada estágio em cada série.

    Esta classe lida com as informações de saída fornecidas pelo
    NWLISTOP e reproduzidas nos `coper.out`.
    """

    T = TypeVar("T")

    BLOCKS = [
        CoperAnos,
    ]

    def __init__(self, data=...) -> None:
        super().__init__(data)
        self.__coper = None

    @classmethod
    def le_arquivo(cls, diretorio: str, nome_arquivo="coper.out") -> "Coper":
        return cls.read(diretorio, nome_arquivo)

    def escreve_arquivo(self, diretorio: str, nome_arquivo="coper.out"):
        self.write(diretorio, nome_arquivo)

    def __monta_tabela(self) -> pd.DataFrame:
        df = None
        for b in self.data.of_type(CoperAnos):
            dados = b.data
            if dados is None:
                continue
            elif df is None:
                df = b.data
            else:
                df = pd.concat([df, b.data], ignore_index=True)
        return df

    @property
    def custos(self) -> Optional[pd.DataFrame]:
        """
        Tabela com os custos por série e
        por mês/ano de estudo.

        - Ano (`int`)
        - Série (`int`)
        - Janeiro (`float`)
        - ...
        - Dezembro (`float`)

        :return: A tabela de custos de operação.
        :rtype: pd.DataFrame | None
        """
        if self.__coper is None:
            self.__coper = self.__monta_tabela()
        return self.__coper



from inewave.nwlistop.earmfpsin import EarmfpSIN

from tests.mocks.mock_open import mock_open
from unittest.mock import MagicMock, patch

from tests.mocks.arquivos.earmfpsin import MockEarmfpsin


def test_atributos_encontrados_earmfpsin():
    m: MagicMock = mock_open(read_data="".join(MockEarmfpsin))
    with patch("builtins.open", m):
        n = EarmfpSIN.le_arquivo("")
        assert n.energias is not None
        assert n.energias.iloc[0, 0] == 2021
        assert n.energias.iloc[-1, -1] == 42.1


def test_atributos_nao_encontrados_earmfpsin():
    m: MagicMock = mock_open(read_data="")
    with patch("builtins.open", m):
        n = EarmfpSIN.le_arquivo("")
        assert n.energias is None


def test_eq_earmfpsin():
    m: MagicMock = mock_open(read_data="".join(MockEarmfpsin))
    with patch("builtins.open", m):
        n1 = EarmfpSIN.le_arquivo("")
        n2 = EarmfpSIN.le_arquivo("")
        assert n1 == n2


# Não deve ter teste de diferença, visto que o atributo é
# implementado como Lazy Property.



from inewave.nwlistop.coper import Coper

from tests.mocks.mock_open import mock_open
from unittest.mock import MagicMock, patch

from tests.mocks.arquivos.coper import MockCoper


def test_atributos_encontrados_coper():
    m: MagicMock = mock_open(read_data="".join(MockCoper))
    with patch("builtins.open", m):
        n = Coper.le_arquivo("")
        assert n.custos is not None
        assert n.custos.iloc[0, 0] == 2022
        assert n.custos.iloc[-1, -1] == 1155.67


def test_atributos_nao_encontrados_coper():
    m: MagicMock = mock_open(read_data="")
    with patch("builtins.open", m):
        n = Coper.le_arquivo("")
        assert n.custos is None


def test_eq_coper():
    m: MagicMock = mock_open(read_data="".join(MockCoper))
    with patch("builtins.open", m):
        n1 = Coper.le_arquivo("")
        n2 = Coper.le_arquivo("")
        assert n1 == n2


# Não deve ter teste de diferença, visto que o atributo é
# implementado como Lazy Property.



from inewave.nwlistop.ghtot00 import Ghtot00

from tests.mocks.mock_open import mock_open
from unittest.mock import MagicMock, patch

from tests.mocks.arquivos.ghtot00 import MockGhtot00


def test_atributos_encontrados_ghtot00():
    m: MagicMock = mock_open(read_data="".join(MockGhtot00))
    with patch("builtins.open", m):
        n = Ghtot00.le_arquivo("")
        assert n.ree is not None
        assert n.ree == "SUDESTE"
        assert n.geracao is not None
        assert n.geracao.iloc[0, 0] == 2022
        assert n.geracao.iloc[-1, -1] == 2578.7


def test_atributos_nao_encontrados_ghtot00():
    m: MagicMock = mock_open(read_data="")
    with patch("builtins.open", m):
        n = Ghtot00.le_arquivo("")
        assert n.ree is None
        assert n.geracao is None


def test_eq_ghtot00():
    m: MagicMock = mock_open(read_data="".join(MockGhtot00))
    with patch("builtins.open", m):
        n1 = Ghtot00.le_arquivo("")
        n2 = Ghtot00.le_arquivo("")
        assert n1 == n2


# Não deve ter teste de diferença, visto que o atributo é
# implementado como Lazy Property.



from inewave.config import MAX_PATAMARES, MESES_DF, MAX_SERIES_SINTETICAS

from cfinterface.components.block import Block
from cfinterface.components.line import Line
from cfinterface.components.field import Field
from cfinterface.components.integerfield import IntegerField
from cfinterface.components.literalfield import LiteralField
from cfinterface.components.floatfield import FloatField
from typing import List, IO
import numpy as np  # type: ignore
import pandas as pd  # type: ignore


class REE(Block):
    """
    Bloco com a informaçao do REE associado aos valores
    de geraçao.
    """

    BEGIN_PATTERN = r"GERACAO HIDRAULICA TOTAL \(MWmes\)"
    END_PATTERN = ""

    def __init__(self, previous=None, next=None, data=None) -> None:
        super().__init__(previous, next, data)
        self.__linha = Line([LiteralField(12, 63)])

    def __eq__(self, o: object) -> bool:
        if not isinstance(o, REE):
            return False
        bloco: REE = o
        if not all(
            [
                isinstance(self.data, str),
                isinstance(o.data, str),
            ]
        ):
            return False
        else:
            return self.data == bloco.data

    # Override
    def read(self, file: IO):
        self.data = self.__linha.read(file.readline())[0]


class GHAnos(Block):
    """
    Bloco com as informações das tabelas de geração hidráulica.
    """

    BEGIN_PATTERN = "     ANO: "
    END_PATTERN = " MEDIA"

    def __init__(self, previous=None, next=None, data=None) -> None:
        super().__init__(previous, next, data)
        self.__linha_ano = Line([IntegerField(4, 10)])
        campos_serie_patamar: List[Field] = [
            IntegerField(4, 2),
            LiteralField(5, 6),
        ]
        campos_custos: List[Field] = [
            FloatField(8, 12 + 9 * i, 1) for i in range(len(MESES_DF) + 1)
        ]
        self.__linha = Line(campos_serie_patamar + campos_custos)

    def __eq__(self, o: object) -> bool:
        if not isinstance(o, GHAnos):
            return False
        bloco: GHAnos = o
        if not all(
            [
                isinstance(self.data, pd.DataFrame),
                isinstance(o.data, pd.DataFrame),
            ]
        ):
            return False
        else:
            return self.data.equals(bloco.data)

    # Override
    def read(self, file: IO):
        def converte_tabela_em_df():
            cols = ["Série"] + MESES_DF + ["Média"]
            df = pd.DataFrame(tabela, columns=cols)
            df["Patamar"] = patamares
            df["Ano"] = self.__ano
            df = df[["Ano", "Série", "Patamar"] + MESES_DF + ["Média"]]
            df = df.astype({"Série": "int64", "Ano": "int64"})
            return df

        self.__ano = self.__linha_ano.read(file.readline())[0]
        file.readline()

        # Variáveis auxiliares
        self.__serie_atual = 0
        tabela = np.zeros(
            (MAX_PATAMARES * MAX_SERIES_SINTETICAS, len(MESES_DF) + 2)
        )
        i = 0
        patamares: List[str] = []
        while True:
            linha = file.readline()
            if self.ends(linha):
                tabela = tabela[:i, :]
                self.data = converte_tabela_em_df()
                break
            dados = self.__linha.read(linha)
            if dados[0] is not None:
                self.__serie_atual = dados[0]
            tabela[i, 0] = self.__serie_atual
            patamares.append(dados[1])
            tabela[i, 1:] = dados[2:]
            i += 1



from inewave.nwlistop.modelos.earmfp00 import REE, EarmsAnos

from cfinterface.files.blockfile import BlockFile
import pandas as pd  # type: ignore
from typing import Type, TypeVar, Optional


class Earmfp00(BlockFile):
    """
    Armazena os dados das saídas referentes às energias
    armazenadas finais, por REE e em % da energia armazenável máxima.

    Esta classe lida com as informações de saída fornecidas pelo
    NWLISTOP e reproduzidas nos `earmfpm00x.out`, onde x varia conforme o
    REE em questão.

    """

    T = TypeVar("T")

    BLOCKS = [
        REE,
        EarmsAnos,
    ]

    def __init__(self, data=...) -> None:
        super().__init__(data)
        self.__earms = None

    @classmethod
    def le_arquivo(
        cls, diretorio: str, nome_arquivo="earmfp001.out"
    ) -> "Earmfp00":
        return cls.read(diretorio, nome_arquivo)

    def escreve_arquivo(self, diretorio: str, nome_arquivo="earmfp001.out"):
        self.write(diretorio, nome_arquivo)

    def __bloco_por_tipo(self, bloco: Type[T], indice: int) -> Optional[T]:
        """
        Obtém um gerador de blocos de um tipo, se houver algum no arquivo.

        :param bloco: Um tipo de bloco para ser lido
        :type bloco: T
        :param indice: O índice do bloco a ser acessado, dentre os do tipo
        :type indice: int
        :return: O gerador de blocos, se houver
        :rtype: Optional[Generator[T], None, None]
        """
        try:
            return next(
                b
                for i, b in enumerate(self.data.of_type(bloco))
                if i == indice
            )
        except StopIteration:
            return None

    def __monta_tabela(self) -> pd.DataFrame:
        df = None
        for b in self.data.of_type(EarmsAnos):
            dados = b.data
            if dados is None:
                continue
            elif df is None:
                df = b.data
            else:
                df = pd.concat([df, b.data], ignore_index=True)
        return df

    @property
    def energias(self) -> Optional[pd.DataFrame]:
        """
        Tabela com as energias armazenadas percentuais por série e
        por mês/ano de estudo.

        - Ano (`int`)
        - Série (`int`)
        - Janeiro (`float`)
        - ...
        - Dezembro (`float`)

        :return: A tabela das energias armazenadas.
        :rtype: pd.DataFrame
        """
        if self.__earms is None:
            self.__earms = self.__monta_tabela()
        return self.__earms

    @property
    def ree(self) -> Optional[str]:
        """
        O REE associado ao arquivo lido.

        :return: Os nome do REE
        :rtype: str
        """
        b = self.__bloco_por_tipo(REE, 0)
        if b is not None:
            return b.data
        return None



# Inclui os membros

from .cmarg00 import Cmarg00  # noqa
from .cmarg00med import Cmarg00med  # noqa
from .coper import Coper  # noqa
from .eafbm00 import Eafbm00  # noqa
from .earmfp00 import Earmfp00  # noqa
from .earmfpm00 import Earmfpm00  # noqa
from .earmfpsin import EarmfpSIN  # noqa
from .ghtot00 import Ghtot00  # noqa
from .ghtotm00 import Ghtotm00  # noqa
from .ghtotsin import GhtotSIN  # noqa
from .mediasmerc import MediasMerc  # noqa
from .mediassin import MediasSIN  # noqa
from .vagua00 import Vagua00  # noqa
from .varmuh import VarmUH  # noqa



from typing import Type, TypeVar, List, Optional, Union
from datetime import datetime

from cfinterface.components.register import Register
from cfinterface.files.registerfile import RegisterFile
from inewave.newave.modelos.eolicageracao import (
    RegistroEolicaGeracaoPeriodo,
    RegistroEolicaGeracaoPatamar,
)


class EolicaGeracao(RegisterFile):
    """
    Armazena os dados de entrada do NEWAVE referentes ao histórico
    de geração das usinas eólicas.
    """

    T = TypeVar("T")

    REGISTERS = [
        RegistroEolicaGeracaoPeriodo,
        RegistroEolicaGeracaoPatamar,
    ]

    def __init__(self, data=...) -> None:
        super().__init__(data)

    @classmethod
    def le_arquivo(
        cls, diretorio: str, nome_arquivo="eolica-geracao.csv"
    ) -> "EolicaGeracao":
        return cls.read(diretorio, nome_arquivo)

    def escreve_arquivo(
        self, diretorio: str, nome_arquivo="eolica-geracao.csv"
    ):
        self.write(diretorio, nome_arquivo)

    def __registros_por_tipo(self, registro: Type[T]) -> List[T]:
        """
        Obtém os registro de um tipo, se houver algum no arquivo.

        :param registro: Um tipo de registro para ser lido
        :type registro: T
        :param indice: O índice do bloco a ser acessado, dentre os do tipo
        :type indice: int

        """
        return [b for b in self.data.of_type(registro)]

    def __obtem_registros(self, tipo: Type[T]) -> List[T]:
        return self.__registros_por_tipo(tipo)

    def __obtem_registros_com_filtros(
        self, tipo_registro: Type[T], **kwargs
    ) -> Optional[Union[T, List[T]]]:
        def __atende(r) -> bool:
            condicoes: List[bool] = []
            for k, v in kwargs.items():
                if v is not None:
                    condicoes.append(getattr(r, k) == v)
            return all(condicoes)

        regs_filtro = [
            r for r in self.__obtem_registros(tipo_registro) if __atende(r)
        ]
        if len(regs_filtro) == 0:
            return None
        elif len(regs_filtro) == 1:
            return regs_filtro[0]
        else:
            return regs_filtro

    def cria_registro(self, anterior: Register, registro: Register):
        """
        Adiciona um registro ao arquivo após um outro registro previamente
        existente.

        Este método existe para retrocompatibilidade e deve ser substituído
        quando for suportado na classe :class:`RegisterFile`.
        """
        self.data.add_after(anterior, registro)

    def deleta_registro(self, registro: Register):
        """
        Remove um registro existente no arquivo.

        Este método existe para retrocompatibilidade e deve ser substituído
        quando for suportado na classe :class:`RegisterFile`.
        """
        self.data.remove(registro)

    def append_registro(self, registro: Register):
        """
        Adiciona um registro ao arquivo na última posição.
        Este método existe para retrocompatibilidade e deve ser substituído
        quando for suportado na classe :class:`RegisterFile`.
        """
        self.data.append(registro)

    def preppend_registro(self, registro: Register):
        """
        Adiciona um registro ao arquivo na primeira posição.
        Este método existe para retrocompatibilidade e deve ser substituído
        quando for suportado na classe :class:`RegisterFile`.
        """
        self.data.preppend(registro)

    def eolica_geracao_periodo(
        self,
        codigo_eolica: Optional[int] = None,
        data_inicial: Optional[datetime] = None,
        data_final: Optional[datetime] = None,
        geracao: Optional[float] = None,
    ) -> Optional[
        Union[
            RegistroEolicaGeracaoPeriodo,
            List[RegistroEolicaGeracaoPeriodo],
        ]
    ]:
        """
        Obtém um registro que contém o valor de geração para uma usina
        eólica durante um período.

        :param codigo_eolica: código da usina eólica
        :type codigo_eolica: int | None
        :param data_inicial: data de início do histórico
        :type data_inicial: datetime | None
        :param data_final: data de fim do histórico
        :type data_final: datetime | None
        :param geracao: valor de geração
        :type geracao: float | None
        :return: Um ou mais registros, se existirem.
        :rtype: :class:`RegistroEolicaGeracaoPeriodo` |
            list[:class:`RegistroEolicaGeracaoPeriodo`] | None
        """
        return self.__obtem_registros_com_filtros(
            RegistroEolicaGeracaoPeriodo,
            codigo_eolica=codigo_eolica,
            data_inicial=data_inicial,
            data_final=data_final,
            geracao=geracao,
        )

    def eolica_geracao_profundidade_periodo_patamar(
        self,
        codigo_eolica: Optional[int] = None,
        data_inicial: Optional[datetime] = None,
        data_final: Optional[datetime] = None,
        indice_patamar: Optional[int] = None,
        profundidade: Optional[float] = None,
    ) -> Optional[
        Union[
            RegistroEolicaGeracaoPatamar,
            List[RegistroEolicaGeracaoPatamar],
        ]
    ]:
        """
        Obtém um registro que contém a profundidade de um patamar
        de geração para um período.

        :param codigo_eolica: código da usina eólica
        :type codigo_eolica: int | None
        :param data_inicial: data de início do registro histórico
        :type data_inicial: datetime | None
        :param data_final: data de fim do registro histórico
        :type data_final: datetime | None
        :param indice_patamar: patamar de geração eólica
        :type indice_patamar: int | None
        :param profundidade: profundidade do patamar de geração
        :type profundidade: float | None
        :return: Um ou mais registros, se existirem.
        :rtype: :class:`RegistroEolicaGeracaoPatamar` |
            list[:class:`RegistroEolicaGeracaoPatamar`] | None
        """
        return self.__obtem_registros_com_filtros(
            RegistroEolicaGeracaoPatamar,
            codigo_eolica=codigo_eolica,
            data_inicial=data_inicial,
            data_final=data_final,
            indice_patamar=indice_patamar,
            profundidade=profundidade,
        )



from inewave.config import MESES_DF, MAX_SERIES_SINTETICAS

from cfinterface.components.block import Block
from cfinterface.components.line import Line
from cfinterface.components.field import Field
from cfinterface.components.integerfield import IntegerField
from cfinterface.components.literalfield import LiteralField
from cfinterface.components.floatfield import FloatField
from typing import List, IO
import numpy as np  # type: ignore
import pandas as pd  # type: ignore


class Submercado(Block):
    """
    Bloco com a informaçao do submercado associado aos valores
    de energia armazenada.
    """

    BEGIN_PATTERN = r"ENERGIA ARMAZENADA FINAL"
    END_PATTERN = ""

    def __init__(self, previous=None, next=None, data=None) -> None:
        super().__init__(previous, next, data)
        self.__linha = Line([LiteralField(12, 70)])

    def __eq__(self, o: object) -> bool:
        if not isinstance(o, Submercado):
            return False
        bloco: Submercado = o
        if not all(
            [
                isinstance(self.data, str),
                isinstance(o.data, str),
            ]
        ):
            return False
        else:
            return self.data == bloco.data

    # Override
    def read(self, file: IO):
        self.data = self.__linha.read(file.readline())[0]


class EarmsAnos(Block):
    """
    Bloco com as informações das tabelas de energias armazenada final
    por série e por mês/ano de estudo.
    """

    BEGIN_PATTERN = "     ANO: "
    END_PATTERN = " MEDIA"

    def __init__(self, previous=None, next=None, data=None) -> None:
        super().__init__(previous, next, data)
        self.__linha_ano = Line([IntegerField(4, 10)])
        campo_serie: List[Field] = [
            IntegerField(4, 2),
        ]
        campos_custos: List[Field] = [
            FloatField(8, 7 + 9 * i, 0) for i in range(len(MESES_DF) + 1)
        ]
        self.__linha = Line(campo_serie + campos_custos)

    def __eq__(self, o: object) -> bool:
        if not isinstance(o, EarmsAnos):
            return False
        bloco: EarmsAnos = o
        if not all(
            [
                isinstance(self.data, pd.DataFrame),
                isinstance(o.data, pd.DataFrame),
            ]
        ):
            return False
        else:
            return self.data.equals(bloco.data)

    # Override
    def read(self, file: IO):
        def converte_tabela_em_df():
            cols = ["Série"] + MESES_DF + ["Média"]
            df = pd.DataFrame(tabela, columns=cols)
            df["Ano"] = self.__ano
            df = df[["Ano"] + cols]
            df = df.astype({"Série": "int64", "Ano": "int64"})
            return df

        self.__ano = self.__linha_ano.read(file.readline())[0]
        file.readline()

        # Variáveis auxiliares
        tabela = np.zeros((MAX_SERIES_SINTETICAS, len(MESES_DF) + 2))
        i = 0
        while True:
            linha = file.readline()
            if self.ends(linha):
                tabela = tabela[:i, :]
                self.data = converte_tabela_em_df()
                break
            tabela[i, :] = self.__linha.read(linha)
            i += 1



from inewave.nwlistop.earmfm00 import Earmfm00

from tests.mocks.mock_open import mock_open
from unittest.mock import MagicMock, patch

from tests.mocks.arquivos.earmfm00 import MockEarmfm00


def test_atributos_encontrados_earmfm00():
    m: MagicMock = mock_open(read_data="".join(MockEarmfm00))
    with patch("builtins.open", m):
        n = Earmfm00.le_arquivo("")
        assert n.energias is not None
        assert n.energias.iloc[0, 0] == 2022
        assert n.energias.iloc[-1, -1] == 122223.0
        assert n.submercado is not None
        assert n.submercado == "SUDESTE"


def test_atributos_nao_encontrados_earmfm00():
    m: MagicMock = mock_open(read_data="")
    with patch("builtins.open", m):
        n = Earmfm00.le_arquivo("")
        assert n.energias is None
        assert n.submercado is None


def test_eq_earmfm00():
    m: MagicMock = mock_open(read_data="".join(MockEarmfm00))
    with patch("builtins.open", m):
        n1 = Earmfm00.le_arquivo("")
        n2 = Earmfm00.le_arquivo("")
        assert n1 == n2


# Não deve ter teste de diferença, visto que o atributo é
# implementado como Lazy Property.



from inewave.nwlistop.cmarg00med import Cmarg00med

from tests.mocks.mock_open import mock_open
from unittest.mock import MagicMock, patch

from tests.mocks.arquivos.cmarg00med import MockCmarg00Med


def test_atributos_encontrados_cmarg00med():
    m: MagicMock = mock_open(read_data="".join(MockCmarg00Med))
    with patch("builtins.open", m):
        n = Cmarg00med.le_arquivo("")
        assert n.custos is not None
        assert n.custos.iloc[0, 0] == 2021
        assert n.custos.iloc[-1, -1] == 354.22
        assert n.submercado is not None
        assert n.submercado == "SUDESTE"


def test_atributos_nao_encontrados_cmarg00med():
    m: MagicMock = mock_open(read_data="")
    with patch("builtins.open", m):
        n = Cmarg00med.le_arquivo("")
        assert n.custos is None
        assert n.submercado is None


def test_eq_cmarg00med():
    m: MagicMock = mock_open(read_data="".join(MockCmarg00Med))
    with patch("builtins.open", m):
        n1 = Cmarg00med.le_arquivo("")
        n2 = Cmarg00med.le_arquivo("")
        assert n1 == n2


# Não deve ter teste de diferença, visto que o atributo é
# implementado como Lazy Property.
